/* $Id: flow.c,v 1.11 2002/10/21 23:22:55 aa5779 Exp $ */
/*
	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <string.h>
#include "util.h"
#include "lists.h"
#include "flow.h"
#include "fflow.h"
#include "pflow.h"
#include "iflow.h"
#include "sflow.h"
#include "dflow.h"
#include "util.h"

static table_t std_flows[] = {{"file", &file_flow_class}, 
							  {"pipe", &pipe_flow_class},
							  {"inline", &inline_flow_class},
							  {"script", &script_flow_class},
							  {"dup", &dup_flow_class},
							  {NULL, NULL}};


DEFLIST(user_flow,
		char *name;
		flow_class *fc;
		);

static user_flow *user_flows;

static int flow_find(user_flow *f, char *key)
{
	return !strcmp(f->name, key);
}

flow_class *detect_flow_class(char *name, char **true_name, char **options)
{
	static char buffer[256];
	user_flow *uf;
	char *end_pref = strstr(name, "://");
	flow_class *res;
	if(!end_pref)
	{
		*true_name = name;
		*options = "";
		return default_flow_class;
	}
	if(end_pref - name >= sizeof(buffer))
		recover_error("flow type id too long");
	memcpy(buffer, name, end_pref - name);
	buffer[end_pref - name] = '\0';
	*true_name = end_pref + 3;
	*options = "";
	if((end_pref = strchr(buffer, '[')))
	{
		*end_pref = '\0';
		*options  = end_pref + 1;
		if((end_pref = strchr(*options, ']')))
			*end_pref = '\0';
		else
			recover_error("expected `]' in flow options");
	}
	uf = list_find(user_flows, buffer, (list_predicate_t)flow_find);
	if(uf)
		res = uf->fc;
	else
	{
		res = lookup_table(std_flows, buffer);
		if(!res)
		{
			res = &script_flow_class;
			if(end_pref)
				(*options)[-1] = '?';
			*options = buffer;
		}
	}
	return res;
}

void register_new_flow(char *name, flow_class *fc)
{
	user_flow *nf = list_find(user_flows, name, (list_predicate_t)flow_find);
	if(nf)
		recover_error("flow type %s already defined", name);
	nf = ALLOC(user_flow);
	nf->name = xstrdup(name);
	nf->fc = fc;
	user_flows = list_add(user_flows, nf);
}
